@@ -40,11 +40,24 @@ class PlaylistModel: MLBaseModel {
     }
 
     func delete(_ items: [VLCMLObject]) {
-        for playlist in items where playlist is VLCMLPlaylist {
+        for case let playlist as VLCMLPlaylist in items {
             if !(medialibrary.deletePlaylist(with: playlist.identifier())) {
                 assertionFailure("PlaylistModel: Failed to delete playlist: \(playlist.identifier())")
             }
+            if playlist.isReadOnly {
+                do {
+                    if let path = playlist.mrl?.path, !path.isEmpty {
+                        try FileManager.default.removeItem(atPath: path)
+                    }
+                } catch let error as NSError {
+                    assertionFailure("PlaylistModel: Delete failed: \(error.localizedDescription)")
+                }
+            }
         }
+
+        // Update directly the UI without waiting the delegate to avoid showing 'ghost' items
+        filterFilesFromDeletion(of: items)
+        updateView?()
     }
 
     // Creates a VLCMLPlaylist appending it and updates linked view
@@ -68,13 +81,6 @@ extension PlaylistModel {
     }
 }
 
-// MARK: - Edit
-extension PlaylistModel: EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type {
-        return MediaEditCell.self
-    }
-}
-
 // MARK: - Search
 extension VLCMLPlaylist: SearchableMLModel {
     func contains(_ searchString: String) -> Bool {
@@ -114,6 +120,10 @@ extension PlaylistModel: MediaLibraryObserver {
         }
         updateView?()
     }
+
+    func medialibraryDidStartRescan() {
+        files.removeAll()
+    }
 }
 
 // MARK: - Helpers
@@ -125,21 +135,6 @@ extension VLCMLPlaylist {
         return String(format: tracksString, mediaCount)
     }
 
-    @objc func thumbnail() -> UIImage? {
-        var image = UIImage(contentsOfFile: artworkMrl())
-        if image == nil {
-            for track in files() ?? [] where track.isThumbnailGenerated() {
-                image = UIImage(contentsOfFile: track.thumbnail()?.path ?? "")
-                break
-            }
-        }
-        if image == nil {
-            let isDarktheme = PresentationTheme.current == PresentationTheme.darkTheme
-            image = isDarktheme ? UIImage(named: "movie-placeholder-dark") : UIImage(named: "movie-placeholder-white")
-        }
-        return image
-    }
-
     func accessibilityText() -> String? {
         return name + " " + numberOfTracksString()
     }
@@ -150,16 +145,12 @@ extension VLCMLPlaylist: MediaCollectionModel {
         return nil
     }
 
-    func files() -> [VLCMLMedia]? {
+    func files(with criteria: VLCMLSortingCriteria = .alpha,
+               desc: Bool = false) -> [VLCMLMedia]? {
         return media
     }
 
     func title() -> String {
         return name
     }
-
-    func sortFilesInCollection(with criteria: VLCMLSortingCriteria,
-                               desc: Bool) -> [VLCMLMedia]? {
-        return nil
-    }
 }
